@@ -19,7 +19,6 @@ cp -r \
   ../../api \
   ../../crypto \
   ../../error \
-  ../../pq-crypto \
   ../../stuffer \
   ../../tls \
   ../../utils \
@@ -48,6 +47,19 @@ cargo publish --dry-run --allow-dirty
 cargo publish --dry-run --allow-dirty --all-features
 popd
 
+# if this version has already been published we can run
+# additional validation to ensure there won't be build
+# problems when a new version is published
+if ! ./scripts/detect-new-release; then
+  pushd s2n-tls
+  cargo publish --dry-run --allow-dirty
+  popd
+
+  pushd s2n-tls-tokio
+  cargo publish --dry-run --allow-dirty
+  popd
+fi
+
 pushd integration
 cargo run
 popd
