@@ -17,20 +17,14 @@
 
 #include "s2n_test.h"
 #include "testlib/s2n_testlib.h"
+#include "tls/s2n_cipher_preferences.h"
+#include "tls/s2n_ktls_parameters.h"
 #include "utils/s2n_random.h"
 #include "utils/s2n_socket.h"
 
 #define S2N_TEST_SEND_FD 66
 #define S2N_TEST_RECV_FD 55
 
-#if defined(S2N_KTLS_SUPPORTED)
-/* It's difficult to test this method via s2n_connection_ktls_enable_send/recv because
- * the key_material is populated via prf, which by definition produces "pseudo-random"
- * output */
-S2N_RESULT s2n_ktls_init_aes128_gcm_crypto_info(struct s2n_connection *conn, s2n_ktls_mode ktls_mode,
-        struct s2n_key_material *key_material, struct tls12_crypto_info_aes_gcm_128 *crypto_info);
-#endif
-
 static int s2n_test_setsockopt_noop(int fd, int level, int optname, const void *optval, socklen_t optlen)
 {
     return S2N_SUCCESS;
@@ -54,15 +48,12 @@ static int s2n_test_setsockopt_tls_error(int fd, int level, int optname, const v
     return 0;
 }
 
-static int s2n_test_setsockopt_tx(int fd, int level, int optname, const void *optval, socklen_t optlen)
+static int s2n_test_setsockopt_aes128_tx(int fd, int level, int optname, const void *optval, socklen_t optlen)
 {
     POSIX_ENSURE_EQ(fd, S2N_TEST_SEND_FD);
-
     if (level == S2N_SOL_TLS) {
         POSIX_ENSURE_EQ(optname, S2N_TLS_TX);
-#if defined(S2N_KTLS_SUPPORTED)
-        POSIX_ENSURE_EQ(optlen, sizeof(struct tls12_crypto_info_aes_gcm_128));
-#endif
+        POSIX_ENSURE_EQ(optlen, sizeof(s2n_ktls_crypto_info_tls12_aes_gcm_128));
     } else if (level == S2N_SOL_TCP) {
         POSIX_ENSURE_EQ(optname, S2N_TCP_ULP);
         POSIX_ENSURE_EQ(optlen, S2N_TLS_ULP_NAME_SIZE);
@@ -72,15 +63,12 @@ static int s2n_test_setsockopt_tx(int fd, int level, int optname, const void *op
     return S2N_SUCCESS;
 }
 
-static int s2n_test_setsockopt_rx(int fd, int level, int optname, const void *optval, socklen_t optlen)
+static int s2n_test_setsockopt_aes128_rx(int fd, int level, int optname, const void *optval, socklen_t optlen)
 {
     POSIX_ENSURE_EQ(fd, S2N_TEST_RECV_FD);
-
     if (level == S2N_SOL_TLS) {
         POSIX_ENSURE_EQ(optname, S2N_TLS_RX);
-#if defined(S2N_KTLS_SUPPORTED)
-        POSIX_ENSURE_EQ(optlen, sizeof(struct tls12_crypto_info_aes_gcm_128));
-#endif
+        POSIX_ENSURE_EQ(optlen, sizeof(s2n_ktls_crypto_info_tls12_aes_gcm_128));
     } else if (level == S2N_SOL_TCP) {
         POSIX_ENSURE_EQ(optname, S2N_TCP_ULP);
         POSIX_ENSURE_EQ(optlen, S2N_TLS_ULP_NAME_SIZE);
@@ -90,7 +78,40 @@ static int s2n_test_setsockopt_rx(int fd, int level, int optname, const void *op
     return S2N_SUCCESS;
 }
 
-S2N_RESULT s2n_test_configure_connection_for_ktls(struct s2n_connection *conn)
+struct s2n_test_setsockopt_expected_struct {
+    size_t count;
+    int fd;
+    int optname;
+    struct s2n_ktls_crypto_info_inputs crypto_info_inputs;
+} s2n_test_setsockopt_expected;
+
+static int s2n_test_setsockopt_tls_cb(int fd, int level,
+        int optname, const void *optval, socklen_t optlen)
+{
+    if (level == S2N_SOL_TLS) {
+        POSIX_ENSURE_EQ(0, s2n_test_setsockopt_expected.count);
+        POSIX_ENSURE_EQ(fd, s2n_test_setsockopt_expected.fd);
+        POSIX_ENSURE_EQ(optname, s2n_test_setsockopt_expected.optname);
+        POSIX_ENSURE_EQ(optval, &s2n_test_setsockopt_expected.crypto_info_inputs);
+        POSIX_ENSURE_EQ(optlen, sizeof(s2n_test_setsockopt_expected.crypto_info_inputs));
+        s2n_test_setsockopt_expected.count++;
+    }
+    return S2N_SUCCESS;
+}
+
+static S2N_RESULT s2n_test_setsockopt_set_ktls_info(struct s2n_ktls_crypto_info_inputs *inputs,
+        struct s2n_ktls_crypto_info *crypto_info)
+{
+    S2N_BLOB_EXPECT_EQUAL(inputs->iv, s2n_test_setsockopt_expected.crypto_info_inputs.iv);
+    S2N_BLOB_EXPECT_EQUAL(inputs->key, s2n_test_setsockopt_expected.crypto_info_inputs.key);
+    S2N_BLOB_EXPECT_EQUAL(inputs->seq, s2n_test_setsockopt_expected.crypto_info_inputs.seq);
+    RESULT_GUARD_POSIX(s2n_blob_init(&crypto_info->value,
+            (void *) &s2n_test_setsockopt_expected.crypto_info_inputs,
+            sizeof(s2n_test_setsockopt_expected.crypto_info_inputs)));
+    return S2N_RESULT_OK;
+}
+
+static S2N_RESULT s2n_test_configure_connection_for_ktls(struct s2n_connection *conn)
 {
     RESULT_ENSURE_REF(conn);
 
@@ -111,33 +132,10 @@ S2N_RESULT s2n_test_configure_connection_for_ktls(struct s2n_connection *conn)
     return S2N_RESULT_OK;
 }
 
-S2N_RESULT s2n_test_generate_fake_crypto_params(struct s2n_connection *conn)
+static int s2n_test_reneg_cb(struct s2n_connection *conn, void *context,
+        s2n_renegotiate_response *response)
 {
-    RESULT_ENSURE_REF(conn);
-
-    struct s2n_blob test_data_blob = { 0 };
-
-    struct s2n_crypto_parameters *server_param = conn->server;
-    RESULT_GUARD_POSIX(s2n_blob_init(&test_data_blob, server_param->server_implicit_iv, sizeof(server_param->server_implicit_iv)));
-    RESULT_GUARD(s2n_get_public_random_data(&test_data_blob));
-    RESULT_GUARD_POSIX(s2n_blob_init(&test_data_blob, server_param->server_sequence_number, sizeof(server_param->server_sequence_number)));
-    RESULT_GUARD(s2n_get_public_random_data(&test_data_blob));
-    RESULT_GUARD_POSIX(s2n_blob_init(&test_data_blob, server_param->client_implicit_iv, sizeof(server_param->client_implicit_iv)));
-    RESULT_GUARD(s2n_get_public_random_data(&test_data_blob));
-    RESULT_GUARD_POSIX(s2n_blob_init(&test_data_blob, server_param->client_sequence_number, sizeof(server_param->client_sequence_number)));
-    RESULT_GUARD(s2n_get_public_random_data(&test_data_blob));
-
-    struct s2n_crypto_parameters *client_param = conn->client;
-    RESULT_GUARD_POSIX(s2n_blob_init(&test_data_blob, client_param->server_implicit_iv, sizeof(client_param->server_implicit_iv)));
-    RESULT_GUARD(s2n_get_public_random_data(&test_data_blob));
-    RESULT_GUARD_POSIX(s2n_blob_init(&test_data_blob, client_param->server_sequence_number, sizeof(client_param->server_sequence_number)));
-    RESULT_GUARD(s2n_get_public_random_data(&test_data_blob));
-    RESULT_GUARD_POSIX(s2n_blob_init(&test_data_blob, client_param->client_implicit_iv, sizeof(client_param->client_implicit_iv)));
-    RESULT_GUARD(s2n_get_public_random_data(&test_data_blob));
-    RESULT_GUARD_POSIX(s2n_blob_init(&test_data_blob, client_param->client_sequence_number, sizeof(client_param->client_sequence_number)));
-    RESULT_GUARD(s2n_get_public_random_data(&test_data_blob));
-
-    return S2N_RESULT_OK;
+    return S2N_SUCCESS;
 }
 
 int main(int argc, char **argv)
@@ -155,22 +153,83 @@ int main(int argc, char **argv)
         END_TEST();
     }
 
-    /* Test ktls_supported ciphers */
+    /* Test set_ktls_info for ciphers */
     {
-        struct s2n_cipher cipher = s2n_aes128_gcm;
-        EXPECT_TRUE(cipher.ktls_supported);
+        struct s2n_crypto_parameters crypto_params = { 0 };
 
-        cipher = s2n_aes256_gcm;
-        EXPECT_FALSE(cipher.ktls_supported);
+        struct s2n_blob test_iv = { 0 };
+        EXPECT_SUCCESS(s2n_blob_init(&test_iv, crypto_params.client_implicit_iv,
+                sizeof(crypto_params.client_implicit_iv)));
+        EXPECT_OK(s2n_get_public_random_data(&test_iv));
 
-        cipher = s2n_tls13_aes128_gcm;
-        EXPECT_FALSE(cipher.ktls_supported);
+        struct s2n_blob test_seq = { 0 };
+        EXPECT_SUCCESS(s2n_blob_init(&test_seq, crypto_params.client_sequence_number,
+                sizeof(crypto_params.client_sequence_number)));
+        EXPECT_OK(s2n_get_public_random_data(&test_seq));
 
-        cipher = s2n_tls13_aes256_gcm;
-        EXPECT_FALSE(cipher.ktls_supported);
+        /* s2n_aes128_gcm */
+        {
+            DEFER_CLEANUP(struct s2n_blob test_key = { 0 }, s2n_free);
+            EXPECT_SUCCESS(s2n_alloc(&test_key, s2n_aes128_gcm.key_material_size));
+            EXPECT_OK(s2n_get_public_random_data(&test_key));
 
-        cipher = s2n_chacha20_poly1305;
-        EXPECT_FALSE(cipher.ktls_supported);
+            struct s2n_ktls_crypto_info_inputs inputs = {
+                .key = test_key,
+                .iv = test_iv,
+                .seq = test_seq,
+            };
+
+            struct s2n_ktls_crypto_info crypto_info = { 0 };
+            EXPECT_OK(s2n_aes128_gcm.set_ktls_info(&inputs, &crypto_info));
+            EXPECT_EQUAL(crypto_info.value.size, sizeof(crypto_info.ciphers.aes_gcm_128));
+            EXPECT_EQUAL(crypto_info.value.data, (uint8_t *) &crypto_info.ciphers.aes_gcm_128);
+            s2n_ktls_crypto_info_tls12_aes_gcm_128 *value =
+                    (s2n_ktls_crypto_info_tls12_aes_gcm_128 *) crypto_info.value.data;
+
+            EXPECT_EQUAL(test_key.size, sizeof(value->key));
+            EXPECT_BYTEARRAY_EQUAL(test_key.data, value->key, sizeof(value->key));
+
+            EXPECT_TRUE(test_iv.size >= sizeof(value->iv));
+            EXPECT_BYTEARRAY_EQUAL(test_iv.data, value->iv, sizeof(value->iv));
+
+            EXPECT_TRUE(test_iv.size >= sizeof(value->salt));
+            EXPECT_BYTEARRAY_EQUAL(test_iv.data, value->salt, sizeof(value->salt));
+
+            EXPECT_TRUE(test_seq.size >= sizeof(value->rec_seq));
+            EXPECT_BYTEARRAY_EQUAL(test_seq.data, value->rec_seq, sizeof(value->rec_seq));
+        };
+
+        /* s2n_aes256_gcm */
+        {
+            DEFER_CLEANUP(struct s2n_blob test_key = { 0 }, s2n_free);
+            EXPECT_SUCCESS(s2n_alloc(&test_key, s2n_aes256_gcm.key_material_size));
+            EXPECT_OK(s2n_get_public_random_data(&test_key));
+
+            struct s2n_ktls_crypto_info_inputs inputs = {
+                .key = test_key,
+                .iv = test_iv,
+                .seq = test_seq,
+            };
+
+            struct s2n_ktls_crypto_info crypto_info = { 0 };
+            EXPECT_OK(s2n_aes256_gcm.set_ktls_info(&inputs, &crypto_info));
+            EXPECT_EQUAL(crypto_info.value.size, sizeof(crypto_info.ciphers.aes_gcm_256));
+            EXPECT_EQUAL(crypto_info.value.data, (uint8_t *) &crypto_info.ciphers.aes_gcm_256);
+            s2n_ktls_crypto_info_tls12_aes_gcm_256 *value =
+                    (s2n_ktls_crypto_info_tls12_aes_gcm_256 *) crypto_info.value.data;
+
+            EXPECT_EQUAL(test_key.size, sizeof(value->key));
+            EXPECT_BYTEARRAY_EQUAL(test_key.data, value->key, sizeof(value->key));
+
+            EXPECT_TRUE(test_iv.size >= sizeof(value->iv));
+            EXPECT_BYTEARRAY_EQUAL(test_iv.data, value->iv, sizeof(value->iv));
+
+            EXPECT_TRUE(test_iv.size >= sizeof(value->salt));
+            EXPECT_BYTEARRAY_EQUAL(test_iv.data, value->salt, sizeof(value->salt));
+
+            EXPECT_TRUE(test_seq.size >= sizeof(value->rec_seq));
+            EXPECT_BYTEARRAY_EQUAL(test_seq.data, value->rec_seq, sizeof(value->rec_seq));
+        };
     };
 
     /* Test s2n_connection_ktls_enable_recv/send */
@@ -181,11 +240,15 @@ int main(int argc, char **argv)
                     s2n_connection_ptr_free);
             EXPECT_OK(s2n_test_configure_connection_for_ktls(server_conn));
 
-            EXPECT_OK(s2n_ktls_set_setsockopt_cb(s2n_test_setsockopt_tx));
+            const struct s2n_cipher *cipher = NULL;
+            EXPECT_OK(s2n_connection_get_secure_cipher(server_conn, &cipher));
+            EXPECT_EQUAL(cipher, &s2n_aes128_gcm);
+
+            EXPECT_OK(s2n_ktls_set_setsockopt_cb(s2n_test_setsockopt_aes128_tx));
             EXPECT_SUCCESS(s2n_connection_ktls_enable_send(server_conn));
             EXPECT_TRUE(server_conn->ktls_send_enabled);
 
-            EXPECT_OK(s2n_ktls_set_setsockopt_cb(s2n_test_setsockopt_rx));
+            EXPECT_OK(s2n_ktls_set_setsockopt_cb(s2n_test_setsockopt_aes128_rx));
             EXPECT_SUCCESS(s2n_connection_ktls_enable_recv(server_conn));
             EXPECT_TRUE(server_conn->ktls_recv_enabled);
         };
@@ -257,7 +320,7 @@ int main(int argc, char **argv)
         {
             /* set kTLS unsupported cipher */
             struct s2n_cipher ktls_temp_unsupported_cipher = {
-                .ktls_supported = false,
+                .set_ktls_info = NULL,
             };
             struct s2n_record_algorithm ktls_temp_unsupported_record_alg = {
                 .cipher = &ktls_temp_unsupported_cipher,
@@ -327,70 +390,134 @@ int main(int argc, char **argv)
             server_conn->managed_recv_io = true;
             EXPECT_SUCCESS(s2n_connection_ktls_enable_recv(server_conn));
         };
-    };
 
-    /* Test s2n_ktls_init_aes128_gcm_crypto_info */
-    {
-        DEFER_CLEANUP(struct s2n_connection *server_conn = s2n_connection_new(S2N_SERVER),
-                s2n_connection_ptr_free);
-        EXPECT_OK(s2n_test_configure_connection_for_ktls(server_conn));
-
-        /* prepare test data */
-        uint8_t test_data[S2N_MAX_KEY_BLOCK_LEN] = { 0 };
-        struct s2n_blob test_data_blob = { 0 };
-        EXPECT_SUCCESS(s2n_blob_init(&test_data_blob, test_data, sizeof(test_data)));
-        EXPECT_OK(s2n_get_public_random_data(&test_data_blob));
-
-        /* copy test data to key_material */
-        struct s2n_key_material key_material = { 0 };
-        EXPECT_OK(s2n_key_material_init(&key_material, server_conn));
-        POSIX_CHECKED_MEMCPY(key_material.key_block, test_data, s2n_array_len(key_material.key_block));
-
-#if defined(S2N_KTLS_SUPPORTED)
-        struct tls12_crypto_info_aes_gcm_128 crypto_info = { 0 };
-
-        /* generate test data for crypto params: implicit_iv and sequence_number */
-        EXPECT_OK(s2n_test_generate_fake_crypto_params(server_conn));
-        struct s2n_crypto_parameters *server_param = server_conn->server;
-
-        /* Test crypto_info for happy case */
+        /* Fail if renegotiation potentially supported */
         {
-            /* server should send with its own keys */
-            int ktls_mode = S2N_KTLS_MODE_SEND;
-            EXPECT_OK(s2n_ktls_init_aes128_gcm_crypto_info(server_conn, ktls_mode, &key_material, &crypto_info));
-            EXPECT_EQUAL(memcmp(crypto_info.key, key_material.server_key.data, key_material.server_key.size), 0);
-            EXPECT_EQUAL(memcmp(crypto_info.iv, server_param->server_implicit_iv, TLS_CIPHER_AES_GCM_128_IV_SIZE), 0);
-            EXPECT_EQUAL(memcmp(crypto_info.salt, server_param->server_implicit_iv, TLS_CIPHER_AES_GCM_128_SALT_SIZE), 0);
-            EXPECT_EQUAL(memcmp(crypto_info.rec_seq, server_param->server_sequence_number, TLS_CIPHER_AES_GCM_128_REC_SEQ_SIZE), 0);
+            DEFER_CLEANUP(struct s2n_config *config = s2n_config_new(), s2n_config_ptr_free);
 
-            /* server should recv with its peer's keys */
-            ktls_mode = S2N_KTLS_MODE_RECV;
-            EXPECT_OK(s2n_ktls_init_aes128_gcm_crypto_info(server_conn, ktls_mode, &key_material, &crypto_info));
-            EXPECT_EQUAL(memcmp(crypto_info.key, key_material.client_key.data, key_material.client_key.size), 0);
-            EXPECT_EQUAL(memcmp(crypto_info.iv, server_param->client_implicit_iv, TLS_CIPHER_AES_GCM_128_IV_SIZE), 0);
-            EXPECT_EQUAL(memcmp(crypto_info.salt, server_param->client_implicit_iv, TLS_CIPHER_AES_GCM_128_SALT_SIZE), 0);
-            EXPECT_EQUAL(memcmp(crypto_info.rec_seq, server_param->client_sequence_number, TLS_CIPHER_AES_GCM_128_REC_SEQ_SIZE), 0);
+            DEFER_CLEANUP(struct s2n_connection *client = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_OK(s2n_test_configure_connection_for_ktls(client));
+            EXPECT_SUCCESS(s2n_connection_set_config(client, config));
 
-            EXPECT_EQUAL(crypto_info.info.cipher_type, TLS_CIPHER_AES_GCM_128);
-            EXPECT_EQUAL(crypto_info.info.version, TLS_1_2_VERSION);
-        };
+            DEFER_CLEANUP(struct s2n_connection *server = s2n_connection_new(S2N_SERVER),
+                    s2n_connection_ptr_free);
+            EXPECT_OK(s2n_test_configure_connection_for_ktls(server));
+            EXPECT_SUCCESS(s2n_connection_set_config(server, config));
 
-        /* Test crypto_info for invalid cases */
+            EXPECT_SUCCESS(s2n_config_set_renegotiate_request_cb(config, s2n_test_reneg_cb, NULL));
+            EXPECT_FAILURE_WITH_ERRNO(s2n_connection_ktls_enable_recv(client), S2N_ERR_KTLS_RENEG);
+            EXPECT_SUCCESS(s2n_connection_ktls_enable_recv(server));
+        }
+
+        /* Call setsockopt correctly to configure tls crypto */
         {
-            int ktls_mode = S2N_KTLS_MODE_SEND;
-            server_conn->actual_protocol_version = S2N_TLS13;
-            EXPECT_ERROR_WITH_ERRNO(s2n_ktls_init_aes128_gcm_crypto_info(server_conn, ktls_mode, &key_material, &crypto_info),
-                    S2N_ERR_KTLS_UNSUPPORTED_CONN);
+            struct s2n_cipher_suite test_cipher_suite = s2n_rsa_with_aes_256_gcm_sha384;
+            struct s2n_record_algorithm test_record_alg = *test_cipher_suite.record_alg;
+            test_cipher_suite.record_alg = &test_record_alg;
+            struct s2n_cipher test_cipher = *test_record_alg.cipher;
+            test_record_alg.cipher = &test_cipher;
+            test_cipher.set_ktls_info = s2n_test_setsockopt_set_ktls_info;
 
-            /* restore protocol version */
-            server_conn->actual_protocol_version = S2N_TLS12;
-            EXPECT_OK(s2n_ktls_init_aes128_gcm_crypto_info(server_conn, ktls_mode, &key_material, &crypto_info));
+            struct s2n_crypto_parameters crypto_params = { 0 };
+            crypto_params.cipher_suite = &test_cipher_suite;
 
-            server_conn->secure->cipher_suite = &s2n_rsa_with_aes_256_gcm_sha384;
-            EXPECT_ERROR_WITH_ERRNO(s2n_ktls_init_aes128_gcm_crypto_info(server_conn, ktls_mode, &key_material, &crypto_info),
-                    S2N_ERR_KTLS_UNSUPPORTED_CONN);
+            struct s2n_blob server_iv = { 0 }, client_iv = { 0 };
+            EXPECT_SUCCESS(s2n_blob_init(&server_iv, crypto_params.server_implicit_iv,
+                    sizeof(crypto_params.server_implicit_iv)));
+            EXPECT_SUCCESS(s2n_blob_init(&client_iv, crypto_params.client_implicit_iv,
+                    sizeof(crypto_params.client_implicit_iv)));
+            EXPECT_OK(s2n_get_public_random_data(&server_iv));
+            EXPECT_OK(s2n_get_public_random_data(&client_iv));
+
+            struct s2n_blob server_seq = { 0 }, client_seq = { 0 };
+            EXPECT_SUCCESS(s2n_blob_init(&server_seq, crypto_params.server_sequence_number,
+                    sizeof(crypto_params.server_sequence_number)));
+            EXPECT_SUCCESS(s2n_blob_init(&client_seq, crypto_params.client_sequence_number,
+                    sizeof(crypto_params.client_sequence_number)));
+            EXPECT_OK(s2n_get_public_random_data(&server_seq));
+            EXPECT_OK(s2n_get_public_random_data(&client_seq));
+
+            /* Test server */
+            {
+                DEFER_CLEANUP(struct s2n_connection *server = s2n_connection_new(S2N_SERVER),
+                        s2n_connection_ptr_free);
+                EXPECT_OK(s2n_test_configure_connection_for_ktls(server));
+                EXPECT_OK(s2n_ktls_set_setsockopt_cb(s2n_test_setsockopt_tls_cb));
+                EXPECT_OK(s2n_crypto_parameters_free(&server->secure));
+                server->secure = &crypto_params;
+
+                struct s2n_key_material key_material = { 0 };
+                EXPECT_OK(s2n_prf_generate_key_material(server, &key_material));
+
+                /* Test server receive */
+                s2n_test_setsockopt_expected = (struct s2n_test_setsockopt_expected_struct){
+                    .fd = S2N_TEST_RECV_FD,
+                    .optname = S2N_TLS_RX,
+                    .crypto_info_inputs = {
+                            .key = key_material.client_key,
+                            .iv = client_iv,
+                            .seq = client_seq,
+                    },
+                };
+                EXPECT_SUCCESS(s2n_connection_ktls_enable_recv(server));
+                EXPECT_EQUAL(s2n_test_setsockopt_expected.count, 1);
+
+                /* Test server send */
+                s2n_test_setsockopt_expected = (struct s2n_test_setsockopt_expected_struct){
+                    .fd = S2N_TEST_SEND_FD,
+                    .optname = S2N_TLS_TX,
+                    .crypto_info_inputs = {
+                            .key = key_material.server_key,
+                            .iv = server_iv,
+                            .seq = server_seq,
+                    },
+                };
+                EXPECT_SUCCESS(s2n_connection_ktls_enable_send(server));
+                EXPECT_EQUAL(s2n_test_setsockopt_expected.count, 1);
+                server->secure = NULL;
+            };
+
+            /* Test client */
+            {
+                DEFER_CLEANUP(struct s2n_connection *client = s2n_connection_new(S2N_CLIENT),
+                        s2n_connection_ptr_free);
+                EXPECT_OK(s2n_test_configure_connection_for_ktls(client));
+                EXPECT_OK(s2n_ktls_set_setsockopt_cb(s2n_test_setsockopt_tls_cb));
+                EXPECT_OK(s2n_crypto_parameters_free(&client->secure));
+                client->secure = &crypto_params;
+
+                struct s2n_key_material key_material = { 0 };
+                EXPECT_OK(s2n_prf_generate_key_material(client, &key_material));
+
+                /* Test client receive */
+                s2n_test_setsockopt_expected = (struct s2n_test_setsockopt_expected_struct){
+                    .fd = S2N_TEST_RECV_FD,
+                    .optname = S2N_TLS_RX,
+                    .crypto_info_inputs = {
+                            .key = key_material.server_key,
+                            .iv = server_iv,
+                            .seq = server_seq,
+                    },
+                };
+                EXPECT_SUCCESS(s2n_connection_ktls_enable_recv(client));
+                EXPECT_EQUAL(s2n_test_setsockopt_expected.count, 1);
+
+                /* Test client send */
+                s2n_test_setsockopt_expected = (struct s2n_test_setsockopt_expected_struct){
+                    .fd = S2N_TEST_SEND_FD,
+                    .optname = S2N_TLS_TX,
+                    .crypto_info_inputs = {
+                            .key = key_material.client_key,
+                            .iv = client_iv,
+                            .seq = client_seq,
+                    },
+                };
+                EXPECT_SUCCESS(s2n_connection_ktls_enable_send(client));
+                EXPECT_EQUAL(s2n_test_setsockopt_expected.count, 1);
+                client->secure = NULL;
+            };
         };
-#endif
     };
 
     /* selftalk: Success case with a real TLS1.2 negotiated server and client */
